from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .content_views_get_response_results_view_model_view import ContentViewsGetResponse_results_view_ModelView
    from .content_views_get_response_results_view_sheet_view import ContentViewsGetResponse_results_view_SheetView

@dataclass
class ContentViewsGetResponse_results_view(Parsable):
    """
    The content view.
    """
    # The 3D model view.
    model_view: Optional[ContentViewsGetResponse_results_view_ModelView] = None
    # The 2D sheet view.
    sheet_view: Optional[ContentViewsGetResponse_results_view_SheetView] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ContentViewsGetResponse_results_view:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ContentViewsGetResponse_results_view
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ContentViewsGetResponse_results_view()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .content_views_get_response_results_view_model_view import ContentViewsGetResponse_results_view_ModelView
        from .content_views_get_response_results_view_sheet_view import ContentViewsGetResponse_results_view_SheetView

        from .content_views_get_response_results_view_model_view import ContentViewsGetResponse_results_view_ModelView
        from .content_views_get_response_results_view_sheet_view import ContentViewsGetResponse_results_view_SheetView

        fields: dict[str, Callable[[Any], None]] = {
            "Model View": lambda n : setattr(self, 'model_view', n.get_object_value(ContentViewsGetResponse_results_view_ModelView)),
            "Sheet View": lambda n : setattr(self, 'sheet_view', n.get_object_value(ContentViewsGetResponse_results_view_SheetView)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("Model View", self.model_view)
        writer.write_object_value("Sheet View", self.sheet_view)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithFormPatchResponse_weather_hourlyWeather(Parsable):
    # Date when weather was first fetched.
    created_at: Optional[datetime.datetime] = None
    # Date when weather was fetched from weather API.
    fetched_at: Optional[datetime.datetime] = None
    # Hour of the day for this forecast.
    hour: Optional[str] = None
    # A percentage value indicating the humidity.
    humidity: Optional[float] = None
    # Unique identifier.
    id: Optional[int] = None
    # Temperature during specified hour.
    temp: Optional[float] = None
    # When the weather was last updated.
    updated_at: Optional[datetime.datetime] = None
    # Direction of the wind, in degrees.
    wind_bearing: Optional[int] = None
    # Average wind speed.
    wind_speed: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithFormPatchResponse_weather_hourlyWeather:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithFormPatchResponse_weather_hourlyWeather
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithFormPatchResponse_weather_hourlyWeather()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "fetchedAt": lambda n : setattr(self, 'fetched_at', n.get_datetime_value()),
            "hour": lambda n : setattr(self, 'hour', n.get_str_value()),
            "humidity": lambda n : setattr(self, 'humidity', n.get_float_value()),
            "id": lambda n : setattr(self, 'id', n.get_int_value()),
            "temp": lambda n : setattr(self, 'temp', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "windBearing": lambda n : setattr(self, 'wind_bearing', n.get_int_value()),
            "windSpeed": lambda n : setattr(self, 'wind_speed', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_datetime_value("fetchedAt", self.fetched_at)
        writer.write_str_value("hour", self.hour)
        writer.write_float_value("humidity", self.humidity)
        writer.write_int_value("id", self.id)
        writer.write_float_value("temp", self.temp)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_int_value("windBearing", self.wind_bearing)
        writer.write_float_value("windSpeed", self.wind_speed)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class RecipientsGetResponse_externalMembers(Parsable):
    # The company name of the external recipient. This value may change if the external user later updates their company information.
    company_name: Optional[str] = None
    # The date and time when the external recipient first downloaded the transmittal, in ISO 8601 format.
    downloaded_at: Optional[datetime.datetime] = None
    # The email address of the external recipient.
    email: Optional[str] = None
    # The name of the external recipient.
    name: Optional[str] = None
    # The date and time when the external recipient received the transmittal notification, in ISO 8601 format.
    received_at: Optional[datetime.datetime] = None
    # The role name of the external recipient.
    role: Optional[str] = None
    # The date and time when the external recipient first viewed the transmittal, in ISO 8601 format.
    viewed_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RecipientsGetResponse_externalMembers:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RecipientsGetResponse_externalMembers
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RecipientsGetResponse_externalMembers()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "companyName": lambda n : setattr(self, 'company_name', n.get_str_value()),
            "downloadedAt": lambda n : setattr(self, 'downloaded_at', n.get_datetime_value()),
            "email": lambda n : setattr(self, 'email', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "receivedAt": lambda n : setattr(self, 'received_at', n.get_datetime_value()),
            "role": lambda n : setattr(self, 'role', n.get_str_value()),
            "viewedAt": lambda n : setattr(self, 'viewed_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("companyName", self.company_name)
        writer.write_datetime_value("downloadedAt", self.downloaded_at)
        writer.write_str_value("email", self.email)
        writer.write_str_value("name", self.name)
        writer.write_datetime_value("receivedAt", self.received_at)
        writer.write_str_value("role", self.role)
        writer.write_datetime_value("viewedAt", self.viewed_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_version_type_query_parameter_type import GetFilterVersionTypeQueryParameterType
    from .get_sort_query_parameter_type import GetSortQueryParameterType
    from .item.with_package_item_request_builder import WithPackageItemRequestBuilder
    from .packages_get_response import PackagesGetResponse

class PackagesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/packages/v1/projects/{projectId}/packages
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PackagesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/packages/v1/projects/{projectId}/packages{?filter%5BcreatedAt%5D*,filter%5BcreatedBy%5D*,filter%5BupdatedAt%5D*,filter%5BupdatedBy%5D*,filter%5BversionType%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_package_id(self,package_id: UUID) -> WithPackageItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.packages.v1.projects.item.packages.item collection
        param package_id: The ID of the package.To find the package ID, call `GET packages </en/docs/acc/v1/reference/http/packages-list-packages-GET/>`_.
        Returns: WithPackageItemRequestBuilder
        """
        if package_id is None:
            raise TypeError("package_id cannot be null.")
        from .item.with_package_item_request_builder import WithPackageItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["packageId"] = package_id
        return WithPackageItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> Optional[PackagesGetResponse]:
        """
        Retrieves a list of all packages within a specified ACC project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PackagesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .packages_get_response import PackagesGetResponse

        return await self.request_adapter.send_async(request_info, PackagesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves a list of all packages within a specified ACC project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> PackagesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: PackagesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return PackagesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class PackagesRequestBuilderGetQueryParameters():
        """
        Retrieves a list of all packages within a specified ACC project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtercreated_at":
                return "filter%5BcreatedAt%5D"
            if original_name == "filtercreated_by":
                return "filter%5BcreatedBy%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filterupdated_by":
                return "filter%5BupdatedBy%5D"
            if original_name == "filterversion_type":
                return "filter%5BversionType%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter packages by their creation time. Use an ISO 8601 date-time range in the format ``startDate..endDate``.Either date may be omitted to specify an open-ended range.Examples:- After a specific time: ``2025-03-26T16:00:00.000Z..``- Before a specific time: ``..2025-03-28T15:59:59.999Z``- Between two times: ``2025-03-26T16:00:00.000Z..2025-03-28T15:59:59.999Z``
        filtercreated_at: Optional[str] = None

        # Filters results by the Autodesk ID of the users who created the packages.You can provide a single Autodesk ID or a comma-separated list of IDs.
        filtercreated_by: Optional[str] = None

        # Filter packages by their last update time. Use an ISO 8601 date-time range in the format ``startDate..endDate``.Either date may be omitted to specify an open-ended range.Examples:- After a specific time: ``2025-03-26T16:00:00.000Z..``- Before a specific time: ``..2025-03-28T15:59:59.999Z``- Between two times: ``2025-03-26T16:00:00.000Z..2025-03-28T15:59:59.999Z``
        filterupdated_at: Optional[str] = None

        # Filters results by the Autodesk ID of the users who last updated the packages.You can provide a single Autodesk ID or a comma-separated list of IDs.To find the IDs call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_
        filterupdated_by: Optional[str] = None

        # Filters results by the version type of the packages.Possible values:- ``FIXED`` – Files in the package remain fixed at selected versions.- ``CURRENT`` – Files in the package automatically update to the latest current versions.For more details, see the `Flexible Package Types <https://help.autodesk.com/view/BUILD/ENU/?guid=File_Packages_Docs>`_ documentation.
        filterversion_type: Optional[GetFilterVersionTypeQueryParameterType] = None

        # The number of packages to return in the response payload.Possible values: ``1-200``. Default: ``200``. For example: ``limit=2``.
        limit: Optional[int] = None

        # The number of packages that you want to begin retrieving results from.Default: ``0``. For example: ``offset=10``
        offset: Optional[int] = None

        # Sorts the results by a supported field.By default, results are sorted in ascending (``asc``) order. To sort in descending order, add ``desc`` after the field name.Format: ``sort=fieldName [desc]``Possible values: ``name``, ``createdAt``, ``updatedAt``, ``displayId``,Examples:- Sort by name (ascending): ``sort=name``- Sort by creation time (descending): ``sort=createdAt desc``
        sort: Optional[GetSortQueryParameterType] = None

    
    @dataclass
    class PackagesRequestBuilderGetRequestConfiguration(RequestConfiguration[PackagesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .item.specs_item_request_builder import SpecsItemRequestBuilder
    from .specs_get_response import SpecsGetResponse
    from .specs_post_request_body import SpecsPostRequestBody
    from .specs_post_response import SpecsPostResponse

class SpecsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/specs
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new SpecsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}/specs{?filter%5Bidentifier%5D*,limit*,offset*,search*,sort*}", path_parameters)
    
    def by_id(self,id: str) -> SpecsItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.submittals.v2.projects.item.specs.item collection
        param id: The ID of the submittal item to retrieve revisions for. To obtain this ID, call `GET items </en/docs/acc/v1/reference/http/submittals-items-GET/>`_.
        Returns: SpecsItemRequestBuilder
        """
        if id is None:
            raise TypeError("id cannot be null.")
        from .item.specs_item_request_builder import SpecsItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["id"] = id
        return SpecsItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SpecsRequestBuilderGetQueryParameters]] = None) -> Optional[SpecsGetResponse]:
        """
        Retrieve all the spec sections for the specified project. For information about spec sections, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Spec_Sections>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SpecsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .specs_get_response import SpecsGetResponse

        return await self.request_adapter.send_async(request_info, SpecsGetResponse, None)
    
    async def post(self,body: SpecsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[SpecsPostResponse]:
        """
        Creates a spec section to organize and categorize submittals.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SpecsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .specs_post_response import SpecsPostResponse

        return await self.request_adapter.send_async(request_info, SpecsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SpecsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieve all the spec sections for the specified project. For information about spec sections, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Spec_Sections>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: SpecsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a spec section to organize and categorize submittals.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> SpecsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SpecsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return SpecsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class SpecsRequestBuilderGetQueryParameters():
        """
        Retrieve all the spec sections for the specified project. For information about spec sections, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Spec_Sections>`_.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteridentifier":
                return "filter%5Bidentifier%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "search":
                return "search"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter spec sections with the specified spec section ID (the spec section ID in the UI). You can specify multiple values. Separate multiple values with commas. For example, ``filter[identifier]=2.``
        filteridentifier: Optional[str] = None

        # The maximum number of results per page. Possible values: ``1``- ``50``. Default value: ``20``. For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The number of results to skip before starting to return data. For example, to skip the first 20 results, include ``offset=20`` in the query string. For more details, see the `JSON API Paging Help documentation <https://jsonapi.org/format/#fetching-pagination>`_.
        offset: Optional[int] = None

        # Search for spec sections by querying a specified string within specific fields (``identifier``, ``title``), and retrieve the associated items that match the search criteria. This includes spec sections where the string matches part of a field. For example, search=1.
        search: Optional[str] = None

        # Sort spec sections by specified fields. Separate multiple values with commas. To sort in descending or ascending order, add ``desc`` or ``asc`` after the sort criteria. For example, ``identifier asc``.Possible values: ``identifier``, ``title``.
        sort: Optional[str] = None

    
    @dataclass
    class SpecsRequestBuilderGetRequestConfiguration(RequestConfiguration[SpecsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class SpecsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_permitted_actions_transitions_state_from import MeGetResponse_permittedActions_transitions_stateFrom
    from .me_get_response_permitted_actions_transitions_state_to import MeGetResponse_permittedActions_transitions_stateTo

@dataclass
class MeGetResponse_permittedActions_transitions(Parsable):
    # Not relevant
    action_id: Optional[str] = None
    # The ID of the transition in the format ``from-state::to-state``. Possible values: ``create::mgr-1``, ``create::sbc-1``.
    id: Optional[str] = None
    # A list of required fields for the transition. For example, [``responseId``].
    mandatory_fields: Optional[list[str]] = None
    # The descriptive name of the transition. Possible values: ``Create``, ``Send to Manager``, ``Send to void``.
    name: Optional[str] = None
    # The starting state of the transition.
    state_from: Optional[MeGetResponse_permittedActions_transitions_stateFrom] = None
    # The target state of the transition.
    state_to: Optional[MeGetResponse_permittedActions_transitions_stateTo] = None
    # Fields that are used in the transition. For example, [``subcontractor``, ``subcontractorType``, ``watchers``, ``responseId``].
    transition_fields: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse_permittedActions_transitions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse_permittedActions_transitions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse_permittedActions_transitions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_permitted_actions_transitions_state_from import MeGetResponse_permittedActions_transitions_stateFrom
        from .me_get_response_permitted_actions_transitions_state_to import MeGetResponse_permittedActions_transitions_stateTo

        from .me_get_response_permitted_actions_transitions_state_from import MeGetResponse_permittedActions_transitions_stateFrom
        from .me_get_response_permitted_actions_transitions_state_to import MeGetResponse_permittedActions_transitions_stateTo

        fields: dict[str, Callable[[Any], None]] = {
            "actionId": lambda n : setattr(self, 'action_id', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "mandatoryFields": lambda n : setattr(self, 'mandatory_fields', n.get_collection_of_primitive_values(str)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "stateFrom": lambda n : setattr(self, 'state_from', n.get_object_value(MeGetResponse_permittedActions_transitions_stateFrom)),
            "stateTo": lambda n : setattr(self, 'state_to', n.get_object_value(MeGetResponse_permittedActions_transitions_stateTo)),
            "transitionFields": lambda n : setattr(self, 'transition_fields', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("actionId", self.action_id)
        writer.write_str_value("id", self.id)
        writer.write_collection_of_primitive_values("mandatoryFields", self.mandatory_fields)
        writer.write_str_value("name", self.name)
        writer.write_object_value("stateFrom", self.state_from)
        writer.write_object_value("stateTo", self.state_to)
        writer.write_collection_of_primitive_values("transitionFields", self.transition_fields)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .asset_statuses.asset_statuses_request_builder import AssetStatusesRequestBuilder
    from .asset_statuses_batch_get.asset_statuses_batch_get_request_builder import AssetStatusesBatchGetRequestBuilder
    from .categories.categories_request_builder import CategoriesRequestBuilder
    from .categories_batch_get.categories_batch_get_request_builder import CategoriesBatchGetRequestBuilder
    from .category_status_step_sets.category_status_step_sets_request_builder import CategoryStatusStepSetsRequestBuilder
    from .custom_attributes.custom_attributes_request_builder import CustomAttributesRequestBuilder
    from .custom_attributes_batch_get.custom_attributes_batch_get_request_builder import CustomAttributesBatchGetRequestBuilder
    from .status_step_sets.status_step_sets_request_builder import StatusStepSetsRequestBuilder
    from .status_step_sets_batch_get.status_step_sets_batch_get_request_builder import StatusStepSetsBatchGetRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v1/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v1/projects/{projectId}", path_parameters)
    
    @property
    def asset_statuses(self) -> AssetStatusesRequestBuilder:
        """
        The assetStatuses property
        """
        from .asset_statuses.asset_statuses_request_builder import AssetStatusesRequestBuilder

        return AssetStatusesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def asset_statuses_batch_get(self) -> AssetStatusesBatchGetRequestBuilder:
        """
        The assetStatusesBatchGet property
        """
        from .asset_statuses_batch_get.asset_statuses_batch_get_request_builder import AssetStatusesBatchGetRequestBuilder

        return AssetStatusesBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def categories(self) -> CategoriesRequestBuilder:
        """
        The categories property
        """
        from .categories.categories_request_builder import CategoriesRequestBuilder

        return CategoriesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def categories_batch_get(self) -> CategoriesBatchGetRequestBuilder:
        """
        The categoriesBatchGet property
        """
        from .categories_batch_get.categories_batch_get_request_builder import CategoriesBatchGetRequestBuilder

        return CategoriesBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def category_status_step_sets(self) -> CategoryStatusStepSetsRequestBuilder:
        """
        The categoryStatusStepSets property
        """
        from .category_status_step_sets.category_status_step_sets_request_builder import CategoryStatusStepSetsRequestBuilder

        return CategoryStatusStepSetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def custom_attributes(self) -> CustomAttributesRequestBuilder:
        """
        The customAttributes property
        """
        from .custom_attributes.custom_attributes_request_builder import CustomAttributesRequestBuilder

        return CustomAttributesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def custom_attributes_batch_get(self) -> CustomAttributesBatchGetRequestBuilder:
        """
        The customAttributesBatchGet property
        """
        from .custom_attributes_batch_get.custom_attributes_batch_get_request_builder import CustomAttributesBatchGetRequestBuilder

        return CustomAttributesBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def status_step_sets(self) -> StatusStepSetsRequestBuilder:
        """
        The statusStepSets property
        """
        from .status_step_sets.status_step_sets_request_builder import StatusStepSetsRequestBuilder

        return StatusStepSetsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def status_step_sets_batch_get(self) -> StatusStepSetsBatchGetRequestBuilder:
        """
        The statusStepSetsBatchGet property
        """
        from .status_step_sets_batch_get.status_step_sets_batch_get_request_builder import StatusStepSetsBatchGetRequestBuilder

        return StatusStepSetsBatchGetRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class WithUser_PatchRequestBody(Parsable):
    # The company_id property
    company_id: Optional[UUID] = None
    # The default_role property
    default_role: Optional[str] = None
    # The status property
    status: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithUser_PatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithUser_PatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithUser_PatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "company_id": lambda n : setattr(self, 'company_id', n.get_uuid_value()),
            "default_role": lambda n : setattr(self, 'default_role', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("company_id", self.company_id)
        writer.write_str_value("default_role", self.default_role)
        writer.write_str_value("status", self.status)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .progress_get_response_results_action_by import ProgressGetResponse_results_actionBy
    from .progress_get_response_results_candidates import ProgressGetResponse_results_candidates
    from .progress_get_response_results_claimed_by import ProgressGetResponse_results_claimedBy
    from .progress_get_response_results_status import ProgressGetResponse_results_status

@dataclass
class ProgressGetResponse_results(Parsable):
    # Information about the user recorded when the step status is ``SUBMITTED`` or ``VOID``. In the Reviews UI, these statuses occur when a participant submits their decision or when a step is voided.
    action_by: Optional[ProgressGetResponse_results_actionBy] = None
    # Information about the users, roles, and companies who are eligible to take the next action in this step.
    candidates: Optional[ProgressGetResponse_results_candidates] = None
    # Information about the user who claimed the step.
    claimed_by: Optional[ProgressGetResponse_results_claimedBy] = None
    # The date and time when the step ended. This value is set when the step is completed, either by being submitted or voided.
    end_time: Optional[datetime.datetime] = None
    # Additional information recorded for this step in the review’s progress.
    notes: Optional[str] = None
    # The current status of the step. Possible values:- ``CLAIMED``: A user has claimed the step.- ``UNCLAIMED``: No user has claimed the step.- ``SUBMITTED``: A user submitted the step (e.g., approved/rejected/custom).- ``VOID``: The step was voided.
    status: Optional[ProgressGetResponse_results_status] = None
    # The ID of the review step this progress record relates to.
    step_id: Optional[str] = None
    # The name of the review step this progress record relates to.
    step_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProgressGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProgressGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProgressGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .progress_get_response_results_action_by import ProgressGetResponse_results_actionBy
        from .progress_get_response_results_candidates import ProgressGetResponse_results_candidates
        from .progress_get_response_results_claimed_by import ProgressGetResponse_results_claimedBy
        from .progress_get_response_results_status import ProgressGetResponse_results_status

        from .progress_get_response_results_action_by import ProgressGetResponse_results_actionBy
        from .progress_get_response_results_candidates import ProgressGetResponse_results_candidates
        from .progress_get_response_results_claimed_by import ProgressGetResponse_results_claimedBy
        from .progress_get_response_results_status import ProgressGetResponse_results_status

        fields: dict[str, Callable[[Any], None]] = {
            "actionBy": lambda n : setattr(self, 'action_by', n.get_object_value(ProgressGetResponse_results_actionBy)),
            "candidates": lambda n : setattr(self, 'candidates', n.get_object_value(ProgressGetResponse_results_candidates)),
            "claimedBy": lambda n : setattr(self, 'claimed_by', n.get_object_value(ProgressGetResponse_results_claimedBy)),
            "endTime": lambda n : setattr(self, 'end_time', n.get_datetime_value()),
            "notes": lambda n : setattr(self, 'notes', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ProgressGetResponse_results_status)),
            "stepId": lambda n : setattr(self, 'step_id', n.get_str_value()),
            "stepName": lambda n : setattr(self, 'step_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("actionBy", self.action_by)
        writer.write_object_value("candidates", self.candidates)
        writer.write_object_value("claimedBy", self.claimed_by)
        writer.write_datetime_value("endTime", self.end_time)
        writer.write_str_value("notes", self.notes)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("stepId", self.step_id)
        writer.write_str_value("stepName", self.step_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .me_get_response_workflow_type import MeGetResponse_workflow_type

@dataclass
class MeGetResponse_workflow(Parsable):
    # The list of RFI workflow roles assigned to the user. Possible values:``projectSC`` — Creator``projectGC`` — Manager``projectCoordinator`` — Reviewer 1 (EMEA workflow only)``projectReviewer`` — Reviewer 1 (US) or Reviewer 2 (EMEA)For information about workflow roles, see the `RFIs Permission <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Statuses>`_ documentation.
    roles: Optional[list[str]] = None
    # The RFI workflow type assigned to the project.Possible values:``US`` Single-reviewer workflow``EU`` Two-reviewer workflowNote that you cannot set the workflow type via the API. To change it, use the Project Admin UI.
    type: Optional[MeGetResponse_workflow_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MeGetResponse_workflow:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MeGetResponse_workflow
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MeGetResponse_workflow()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .me_get_response_workflow_type import MeGetResponse_workflow_type

        from .me_get_response_workflow_type import MeGetResponse_workflow_type

        fields: dict[str, Callable[[Any], None]] = {
            "roles": lambda n : setattr(self, 'roles', n.get_collection_of_primitive_values(str)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(MeGetResponse_workflow_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("roles", self.roles)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .relationships_sync_post_response_current import RelationshipsSyncPostResponse_current
    from .relationships_sync_post_response_deleted import RelationshipsSyncPostResponse_deleted

@dataclass
class RelationshipsSyncPostResponse(Parsable):
    # A set of current data.
    current: Optional[list[RelationshipsSyncPostResponse_current]] = None
    # A set of deleted data.
    deleted: Optional[list[RelationshipsSyncPostResponse_deleted]] = None
    # If set to true, data is available for synchronization using the supplied synchronization token.
    more_data: Optional[bool] = None
    # The token that can be used to obtain data via the synchronization endpoint. If moreData is set to false, you can use this token to resume synchronization at a later point in time.
    next_sync_token: Optional[str] = None
    # If set to true, the data returned by the synchronization endpoint can be used to overwrite local copies.
    overwrite: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RelationshipsSyncPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RelationshipsSyncPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RelationshipsSyncPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .relationships_sync_post_response_current import RelationshipsSyncPostResponse_current
        from .relationships_sync_post_response_deleted import RelationshipsSyncPostResponse_deleted

        from .relationships_sync_post_response_current import RelationshipsSyncPostResponse_current
        from .relationships_sync_post_response_deleted import RelationshipsSyncPostResponse_deleted

        fields: dict[str, Callable[[Any], None]] = {
            "current": lambda n : setattr(self, 'current', n.get_collection_of_object_values(RelationshipsSyncPostResponse_current)),
            "deleted": lambda n : setattr(self, 'deleted', n.get_collection_of_object_values(RelationshipsSyncPostResponse_deleted)),
            "moreData": lambda n : setattr(self, 'more_data', n.get_bool_value()),
            "nextSyncToken": lambda n : setattr(self, 'next_sync_token', n.get_str_value()),
            "overwrite": lambda n : setattr(self, 'overwrite', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("current", self.current)
        writer.write_collection_of_object_values("deleted", self.deleted)
        writer.write_bool_value("moreData", self.more_data)
        writer.write_str_value("nextSyncToken", self.next_sync_token)
        writer.write_bool_value("overwrite", self.overwrite)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .patch_request_body_additional_collaborators import PatchRequestBody_additionalCollaborators

@dataclass
class PatchRequestBody(Parsable):
    # The actual completion date of the main contract.
    actual_completion_date: Optional[str] = None
    # The additional collaborator company and contacts.
    additional_collaborators: Optional[list[PatchRequestBody_additionalCollaborators]] = None
    # The BIM360/ACC ID of the architecture firm.
    architect_company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    architect_company_uid: Optional[str] = None
    # The BIM360/ACC ID of the primary contact at the architecture firm.
    architect_contact_id: Optional[str] = None
    # The closing date of the main contract.
    close_date: Optional[str] = None
    # The code of the main contract.Max length: 255
    code: Optional[str] = None
    # The completed work retention percentage of the main contract amount.
    completed_work_retention_percent: Optional[float] = None
    # The BIM360/ACC ID of the supplier's default contact.
    contact_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    contractor_company_uid: Optional[str] = None
    # Detailed description of a main contract.Max length: 2048
    description: Optional[str] = None
    # The execution date of the main contract.
    executed_date: Optional[str] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # ``true``: This is the default main contract.``false``: This is not the default main contract.
    is_default: Optional[bool] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The materials retention percentage of the main contract amount.
    materials_retention_percent: Optional[float] = None
    # The name of the main contract.Max length: 1024
    name: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    notary_company_uid: Optional[str] = None
    # A note about the main contract in `Tiptap <https://tiptap.dev/introduction/>`_-formatted rich text.
    note: Optional[str] = None
    # The BIM360/ACC ID of the owner company.
    owner_company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    owner_company_uid: Optional[str] = None
    # The BIM360/ACC ID of the primary contact at the owner company.
    owner_contact_id: Optional[str] = None
    # The planned completion date of the main contract.
    planned_completion_date: Optional[str] = None
    # Total retention percentage of the main contract amount.
    retention_cap: Optional[float] = None
    # The revised completion date of the main contract.
    revised_completion_date: Optional[str] = None
    # The scope of work signed by all parties to the main contract, in `Tiptap <https://tiptap.dev/introduction/>`_-formatted rich text.
    scope_of_work: Optional[str] = None
    # The start date of the main contract.
    start_date: Optional[str] = None
    # The status of the main contract. Possible values: ``closed``, ``executed``, ``review``, ``signed``.
    status: Optional[str] = None
    # The type of the main contract. For example, ``Fixed Price`` or ``Unit Price``. This is customizable for the project administrator.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .patch_request_body_additional_collaborators import PatchRequestBody_additionalCollaborators

        from .patch_request_body_additional_collaborators import PatchRequestBody_additionalCollaborators

        fields: dict[str, Callable[[Any], None]] = {
            "actualCompletionDate": lambda n : setattr(self, 'actual_completion_date', n.get_str_value()),
            "additionalCollaborators": lambda n : setattr(self, 'additional_collaborators', n.get_collection_of_object_values(PatchRequestBody_additionalCollaborators)),
            "architectCompanyId": lambda n : setattr(self, 'architect_company_id', n.get_str_value()),
            "architectCompanyUid": lambda n : setattr(self, 'architect_company_uid', n.get_str_value()),
            "architectContactId": lambda n : setattr(self, 'architect_contact_id', n.get_str_value()),
            "closeDate": lambda n : setattr(self, 'close_date', n.get_str_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "completedWorkRetentionPercent": lambda n : setattr(self, 'completed_work_retention_percent', n.get_float_value()),
            "contactId": lambda n : setattr(self, 'contact_id', n.get_str_value()),
            "contractorCompanyUid": lambda n : setattr(self, 'contractor_company_uid', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "executedDate": lambda n : setattr(self, 'executed_date', n.get_str_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "isDefault": lambda n : setattr(self, 'is_default', n.get_bool_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "materialsRetentionPercent": lambda n : setattr(self, 'materials_retention_percent', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "notaryCompanyUid": lambda n : setattr(self, 'notary_company_uid', n.get_str_value()),
            "note": lambda n : setattr(self, 'note', n.get_str_value()),
            "ownerCompanyId": lambda n : setattr(self, 'owner_company_id', n.get_str_value()),
            "ownerCompanyUid": lambda n : setattr(self, 'owner_company_uid', n.get_str_value()),
            "ownerContactId": lambda n : setattr(self, 'owner_contact_id', n.get_str_value()),
            "plannedCompletionDate": lambda n : setattr(self, 'planned_completion_date', n.get_str_value()),
            "retentionCap": lambda n : setattr(self, 'retention_cap', n.get_float_value()),
            "revisedCompletionDate": lambda n : setattr(self, 'revised_completion_date', n.get_str_value()),
            "scopeOfWork": lambda n : setattr(self, 'scope_of_work', n.get_str_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("actualCompletionDate", self.actual_completion_date)
        writer.write_collection_of_object_values("additionalCollaborators", self.additional_collaborators)
        writer.write_str_value("architectCompanyId", self.architect_company_id)
        writer.write_str_value("architectCompanyUid", self.architect_company_uid)
        writer.write_str_value("architectContactId", self.architect_contact_id)
        writer.write_str_value("closeDate", self.close_date)
        writer.write_str_value("code", self.code)
        writer.write_float_value("completedWorkRetentionPercent", self.completed_work_retention_percent)
        writer.write_str_value("contactId", self.contact_id)
        writer.write_str_value("contractorCompanyUid", self.contractor_company_uid)
        writer.write_str_value("description", self.description)
        writer.write_str_value("executedDate", self.executed_date)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_bool_value("isDefault", self.is_default)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_float_value("materialsRetentionPercent", self.materials_retention_percent)
        writer.write_str_value("name", self.name)
        writer.write_str_value("notaryCompanyUid", self.notary_company_uid)
        writer.write_str_value("note", self.note)
        writer.write_str_value("ownerCompanyId", self.owner_company_id)
        writer.write_str_value("ownerCompanyUid", self.owner_company_uid)
        writer.write_str_value("ownerContactId", self.owner_contact_id)
        writer.write_str_value("plannedCompletionDate", self.planned_completion_date)
        writer.write_float_value("retentionCap", self.retention_cap)
        writer.write_str_value("revisedCompletionDate", self.revised_completion_date)
        writer.write_str_value("scopeOfWork", self.scope_of_work)
        writer.write_str_value("startDate", self.start_date)
        writer.write_str_value("status", self.status)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .requirements.requirements_request_builder import RequirementsRequestBuilder
    from .smartregister.smartregister_request_builder import SmartregisterRequestBuilder
    from .submittals_summary.submittals_summary_request_builder import SubmittalsSummaryRequestBuilder

class WithVersionItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/autospecs/v1/projects/{projectId}/version/{versionId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithVersionItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/autospecs/v1/projects/{projectId}/version/{versionId}", path_parameters)
    
    @property
    def requirements(self) -> RequirementsRequestBuilder:
        """
        The requirements property
        """
        from .requirements.requirements_request_builder import RequirementsRequestBuilder

        return RequirementsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def smartregister(self) -> SmartregisterRequestBuilder:
        """
        The smartregister property
        """
        from .smartregister.smartregister_request_builder import SmartregisterRequestBuilder

        return SmartregisterRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def submittals_summary(self) -> SubmittalsSummaryRequestBuilder:
        """
        The submittalsSummary property
        """
        from .submittals_summary.submittals_summary_request_builder import SubmittalsSummaryRequestBuilder

        return SubmittalsSummaryRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.with_job_item_request_builder import WithJobItemRequestBuilder
    from .jobs_get_response import JobsGetResponse

class JobsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /data-connector/v1/accounts/{accountId}/jobs
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new JobsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/data-connector/v1/accounts/{accountId}/jobs{?filter%5Bfield_to_filter%5D*,limit*,offset*,projectId*,sort*,sortFields*}", path_parameters)
    
    def by_job_id(self,job_id: UUID) -> WithJobItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.dataConnector.v1.accounts.item.jobs.item collection
        param job_id: The job ID
        Returns: WithJobItemRequestBuilder
        """
        if job_id is None:
            raise TypeError("job_id cannot be null.")
        from .item.with_job_item_request_builder import WithJobItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["jobId"] = job_id
        return WithJobItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[JobsRequestBuilderGetQueryParameters]] = None) -> Optional[JobsGetResponse]:
        """
        To get request IDs for your requests, use `GET requests </en/docs/bim360/v1/reference/http/data-connector-requests-GET/>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[JobsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .jobs_get_response import JobsGetResponse

        return await self.request_adapter.send_async(request_info, JobsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[JobsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        To get request IDs for your requests, use `GET requests </en/docs/bim360/v1/reference/http/data-connector-requests-GET/>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> JobsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: JobsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return JobsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class JobsRequestBuilderGetQueryParameters():
        """
        To get request IDs for your requests, use `GET requests </en/docs/bim360/v1/reference/http/data-connector-requests-GET/>`_.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterfield_to_filter":
                return "filter%5Bfield_to_filter%5D"
            if original_name == "project_id":
                return "projectId"
            if original_name == "sort_fields":
                return "sortFields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Return only the Data Connector jobs in which the specified field has the specified value. Use the following format in the endpoint URL: ``filter[field_to_filter]=_filter_value_``.Note that you can provide multiple job filters in the URL, but you may not filter on multiple values of the same field.Possible ``field_to_filter`` values: ``projectId``, ``createdAt``, ``status``, ``completionStatus``, ``startedAt``, ``completedAt``, ``startDate``, ``endDate``. You can also retrieve all Data Connector jobs in the current account by specifying ``filter[projectId]=null``.For the ``createdAt``, ``startedAt``, and ``completedAt`` fields (which accept a date range), specify the range in the form *firstdate..lastdate*, in ISO 8601 format with the time required. You can alternatively omit the first date (for example, to get everything on or before June 1, 2019 the range would be ``..2019-06-01T23:59:59.999Z``). You can also omit the last date (for example, to get everything after June 1, 2019 the range would be ``2019-06-01T00:00:00.000Z..``)Additional notes on using ``startDate`` and ``endDate``:- If you provide only ``startDate`` or ``endDate`` (but not both), Data Connector uses that single date for both ``startDate`` and ``endDate``.- If you request more than the Maximum Date Range Allowed for an extraction, the default date range as documented in the schema documentation is returned.- For the ``activities`` service group, data replication can be delayed up to 20 minutes, so your requests should account for that delay.
        filterfield_to_filter: Optional[str] = None

        # The number of data connector objects to return. Default value: 20
        limit: Optional[int] = None

        # The number of data objects to skip before starting to starting to collect the result set. Default value: 0
        offset: Optional[int] = None

        # Project ID of the returned Data Connector objects.
        project_id: Optional[str] = None

        # The sort order of returned data connector objects. Possible values: ``asc`` ascending by date (earliest to latest date), ``desc`` descending by date (latest to earliest date).
        sort: Optional[str] = None

        # A string of comma-separated names of the fields by which to sort the returned jobs. The results are sorted by the first field, then by the second field, and so on. Jobs are sorted by each field in ascending order by default; you can prepend any field name with a hyphen ( ``-`` ) to sort in descending order. Invalid fields and whitespaces are ignored.Possible values: ``projectId``, ``createdBy``, ``createdByEmail``, ``createdAt``, ``status``, ``completionStatus``, ``startedAt``, ``completedAt``, ``startDate``, ``endDate``.
        sort_fields: Optional[str] = None

    
    @dataclass
    class JobsRequestBuilderGetRequestConfiguration(RequestConfiguration[JobsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_text_match_query_parameter_type import GetFilterTextMatchQueryParameterType
    from .roles_get_response import RolesGetResponse

class RolesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/accounts/{accountId}/users/{userId}/roles
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new RolesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/accounts/{accountId}/users/{userId}/roles{?fields*,filterTextMatch*,filter%5Bname%5D*,filter%5BprojectId%5D*,filter%5Bstatus%5D*,limit*,offset*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[RolesRequestBuilderGetQueryParameters]] = None) -> Optional[RolesGetResponse]:
        """
        Returns the roles assigned to a specific user across the projects they belong to.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[RolesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .roles_get_response import RolesGetResponse

        return await self.request_adapter.send_async(request_info, RolesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[RolesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Returns the roles assigned to a specific user across the projects they belong to.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> RolesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: RolesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return RolesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class RolesRequestBuilderGetQueryParameters():
        """
        Returns the roles assigned to a specific user across the projects they belong to.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filterproject_id":
                return "filter%5BprojectId%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filter_text_match":
                return "filterTextMatch"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # A comma-separated list of response fields to include.Defaults to all fields if not specified.Use this parameter to reduce the response size by retrieving only the fields you need.Possible values:``projectIds`` – Projects where the user holds this role``name`` – Role name``status`` – Role status (active or inactive)``key`` – Internal key used to translate the role name``createdAt`` – Timestamp when the role was created``updatedAt`` – Timestamp when the role was last updated
        fields: Optional[list[str]] = None

        # Specifies how text-based filters should match values in supported fields.This parameter can be used in any endpoint that supports text-based filtering (e.g., ``filter[name]``, ``filter[jobNumber]``, ``filter[companyName]``, etc.).Possible values:``contains`` (default) – Matches if the field contains the specified text anywhere``startsWith`` – Matches if the field starts with the specified text``endsWith`` – Matches if the field ends with the specified text``equals`` – Matches only if the field exactly matches the specified textMatching is case-insensitive.Wildcards and regular expressions are not supported.
        filter_text_match: Optional[GetFilterTextMatchQueryParameterType] = None

        # Filters roles by name.By default, this performs a partial match (case-insensitive).You can control how the match behaves by using the ``filterTextMatch`` parameter. For example, to match only names that start with (`startsWith`), end with (`endsWith`), or exactly equal (`equals`) the provided value.
        filtername: Optional[str] = None

        # A list of project IDs. Only results where the user is associated with one or more of the specified projects are returned.
        filterproject_id: Optional[list[str]] = None

        # Filters roles by their status. Accepts one or more of the following values:``active`` – The role is currently in use.``inactive`` – The role has been removed or is no longer in use.
        filterstatus: Optional[list[str]] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # Sorts the results by one or more fields.Each field can be followed by a direction modifier:``asc`` – Ascending order (default)``desc`` – Descending orderPossible values: ``name``, ``createdAt``, ``updatedAt``.Default sort: ``name asc``Example: ``sort=name,updatedAt desc``
        sort: Optional[list[str]] = None

    
    @dataclass
    class RolesRequestBuilderGetRequestConfiguration(RequestConfiguration[RolesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ItemsPatchRequestBody(Parsable):
    # The total price of the main contract item.
    amount: Optional[float] = None
    # A code for the main contract item.Max length: 255
    code: Optional[str] = None
    # A description of the main contract item.Max length: 2048
    description: Optional[str] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The name of the main contract item field to lock so the API can use it to calculate the value of a related field. For example, you can lock the ``unitPrice`` field in the existing main contract item, and the API will use the new value of ``quantity`` that you provide to calculate the new value of ``amount``.Possible values: ``amount``, ``quantity``, and ``unitPrice``.
    locked_field: Optional[str] = None
    # A name for the main contract item.Max length: 1024
    name: Optional[str] = None
    # The position of the main contract item in its siblings of main contract items.
    position: Optional[int] = None
    # The quantity of the main contract item.
    quantity: Optional[float] = None
    # The unit of measure of the main contract item.Max length: 255
    unit: Optional[str] = None
    # The unit price of the main contract item.
    unit_price: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ItemsPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ItemsPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ItemsPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "lockedField": lambda n : setattr(self, 'locked_field', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "position": lambda n : setattr(self, 'position', n.get_int_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("amount", self.amount)
        writer.write_str_value("code", self.code)
        writer.write_str_value("description", self.description)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_str_value("lockedField", self.locked_field)
        writer.write_str_value("name", self.name)
        writer.write_int_value("position", self.position)
        writer.write_float_value("quantity", self.quantity)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .search_rfis_post_request_body_filter import SearchRfisPostRequestBody_filter
    from .search_rfis_post_request_body_sort import SearchRfisPostRequestBody_sort

@dataclass
class SearchRfisPostRequestBody(Parsable):
    # Specify which attributes to include in the response.Use this to limit the response to only the fields you need.For example, ``fields=id, title``.
    fields: Optional[list[str]] = None
    # A set of optional filters to narrow the results. You can combine multiple filters.
    filter: Optional[list[SearchRfisPostRequestBody_filter]] = None
    # The number of RFIs to return. Default: ``10``. Maximum: ``200``.
    limit: Optional[int] = None
    # The number of items to skip before starting the result set. Default: ``0``.
    offset: Optional[int] = None
    # Searches for a string in the ``title``, ``question``, and ``officialResponse`` fields.
    search: Optional[str] = None
    # A list of sort rules to apply. Each item includes a ``field`` to sort by and the sort ``order``.
    sort: Optional[list[SearchRfisPostRequestBody_sort]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SearchRfisPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SearchRfisPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SearchRfisPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .search_rfis_post_request_body_filter import SearchRfisPostRequestBody_filter
        from .search_rfis_post_request_body_sort import SearchRfisPostRequestBody_sort

        from .search_rfis_post_request_body_filter import SearchRfisPostRequestBody_filter
        from .search_rfis_post_request_body_sort import SearchRfisPostRequestBody_sort

        fields: dict[str, Callable[[Any], None]] = {
            "fields": lambda n : setattr(self, 'fields', n.get_collection_of_primitive_values(str)),
            "filter": lambda n : setattr(self, 'filter', n.get_collection_of_object_values(SearchRfisPostRequestBody_filter)),
            "limit": lambda n : setattr(self, 'limit', n.get_int_value()),
            "offset": lambda n : setattr(self, 'offset', n.get_int_value()),
            "search": lambda n : setattr(self, 'search', n.get_str_value()),
            "sort": lambda n : setattr(self, 'sort', n.get_collection_of_object_values(SearchRfisPostRequestBody_sort)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("fields", self.fields)
        writer.write_collection_of_object_values("filter", self.filter)
        writer.write_int_value("limit", self.limit)
        writer.write_int_value("offset", self.offset)
        writer.write_str_value("search", self.search)
        writer.write_collection_of_object_values("sort", self.sort)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .search_rfis_post_request_body_filter_custom_attributes import SearchRfisPostRequestBody_filter_customAttributes

@dataclass
class SearchRfisPostRequestBody_filter(Parsable):
    # Filter RFIs assigned to specific users.To find the ID call `GET users/me </en/docs/bim360/v1/reference/http/rfis-users-me-GET/>`_ and check ``user.id``.
    assigned_to: Optional[list[str]] = None
    # Filter RFIs by category.Categories are customizable per project. Examples include ``Constructability``, ``Code Compliance``. For example, ``filter[category]=Constructability``.Possible values: ``Code Compliance``, ``Constructability``, ``Design Coordination``, ``Documentation Conflict``, ``Documentation Incomplete``, ``Field condition``, ``Other``, ``Unspecified``.
    category: Optional[list[str]] = None
    # Filter RFIs by close date or date range.Format: ``YYYY-MM-DDThh:mm:ss.sz..YYYY-MM-DDThh:mm:ss.sz``.
    closed_at: Optional[str] = None
    # Filter RFIs by cost impact value.Possible values: ``Yes``, ``No``, ``Unknown``. For example, ``filter[costImpact]=Yes``'
    cost_impact: Optional[list[str]] = None
    # Filter RFIs created after a specific date or within a date range.Format: ``YYYY-MM-DDThh:mm:ss.sz..YYYY-MM-DDThh:mm:ss.sz``.
    created_at: Optional[str] = None
    # Filter RFIs by the user who created them.To find the ID call `GET users/me </en/docs/bim360/v1/reference/http/rfis-users-me-GET/>`_ and check ``user.id``.``
    created_by: Optional[list[str]] = None
    # Filter RFIs by custom attributes.Provide a map of key-value pairs using attribute ID and selected value ID.For example, ``fd9a1234-aaaa-4444-bbbb-8888aa77ee66: value-id-1``.
    custom_attributes: Optional[SearchRfisPostRequestBody_filter_customAttributes] = None
    # Filter RFIs by discipline.Values depend on the project configuration. Examples include ``Architectural``. For example, ``filter[discipline]=Architectural``.Possible values: ``Building Management System``, ``Electrical Substation``, ``Security``, ``Audio Visual``, ``Food Service``, ``Fire Alarm``, ``Power Systems``, ``Design Systems Integrator``, ``Signage``, ``Pathways``, ``Cabling``, ``Networks``, ``Distributed Antenna System``, ``Lighting``, ``Vertical Transportation``, ``Roofing``, ``Architectural``, ``Civil/Site``, ``Concrete``, ``Electrical``, ``Exterior Envelope``, ``Fire Protection``, ``Interior/Finishes``, ``Landscaping``, ``Masonry``, ``Mechanical``, ``Plumbing``, ``Structural``, ``Other``, ``Unspecified``.
    discipline: Optional[list[str]] = None
    # Filter RFIs by due date or due date range.Format: ``YYYY-MM-DDThh:mm:ss.sz..YYYY-MM-DDThh:mm:ss.sz``.
    due_date: Optional[str] = None
    # Filter by a list of specific RFI IDs.To find the RFI IDs call `GET rfis </en/docs/bim360/v1/reference/http/rfis-v2-rfis-GET/>`_ and check ``results.id``.
    id: Optional[list[str]] = None
    # ``true``: include RFIs that are hidden from the user.``false``: (default) exclude hidden RFIs from the results.
    include_hidden: Optional[bool] = None
    # Filter RFIs by location using Location Breakdown Structure (LBS) node IDs.To retrieve location IDs, call `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.'
    locations: Optional[list[str]] = None
    # Filters RFIs by the official response status. Possible values: ``answered``, ``unanswered``.
    official_response_status: Optional[list[str]] = None
    # Filter RFIs by priority level.Possible values: ``High``, ``Normal``, ``Low``. For example, ``filter[priority]=High``.'
    priority: Optional[list[str]] = None
    # Filter RFIs by the external reference ID (e.g., model number or spec reference).Max length: 20 characters.Max length: 20
    reference: Optional[str] = None
    # Filter RFIs by RFI type ID.RFI types are defined at the project level. Use UUIDs returned from the project configuration.
    rfi_type_id: Optional[list[str]] = None
    # Filter RFIs by schedule impact value.Possible values**: ``Yes``, ``No``, ``Unknown``. For example, ``filter[scheduleImpact]=Yes``'
    schedule_impact: Optional[list[str]] = None
    # Filter by RFI status. Possible values: ``draft``, ``submitted``, ``open``, ``openRev1``, ``openRev2``, answered, ``answeredRev1``, ``answeredManager``, ``closed``, ``void``. Note that workflow types determine which statuses are used.
    status: Optional[list[str]] = None
    # Filter RFIs updated after a specific date, or within a date range.Format: ``YYYY-MM-DDThh:mm:ss.sz..YYYY-MM-DDThh:mm:ss.sz``.
    updated_at: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SearchRfisPostRequestBody_filter:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SearchRfisPostRequestBody_filter
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SearchRfisPostRequestBody_filter()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .search_rfis_post_request_body_filter_custom_attributes import SearchRfisPostRequestBody_filter_customAttributes

        from .search_rfis_post_request_body_filter_custom_attributes import SearchRfisPostRequestBody_filter_customAttributes

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_collection_of_primitive_values(str)),
            "category": lambda n : setattr(self, 'category', n.get_collection_of_primitive_values(str)),
            "closedAt": lambda n : setattr(self, 'closed_at', n.get_str_value()),
            "costImpact": lambda n : setattr(self, 'cost_impact', n.get_collection_of_primitive_values(str)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_collection_of_primitive_values(str)),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_object_value(SearchRfisPostRequestBody_filter_customAttributes)),
            "discipline": lambda n : setattr(self, 'discipline', n.get_collection_of_primitive_values(str)),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_collection_of_primitive_values(str)),
            "includeHidden": lambda n : setattr(self, 'include_hidden', n.get_bool_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "officialResponseStatus": lambda n : setattr(self, 'official_response_status', n.get_collection_of_primitive_values(str)),
            "priority": lambda n : setattr(self, 'priority', n.get_collection_of_primitive_values(str)),
            "reference": lambda n : setattr(self, 'reference', n.get_str_value()),
            "rfiTypeId": lambda n : setattr(self, 'rfi_type_id', n.get_collection_of_primitive_values(str)),
            "scheduleImpact": lambda n : setattr(self, 'schedule_impact', n.get_collection_of_primitive_values(str)),
            "status": lambda n : setattr(self, 'status', n.get_collection_of_primitive_values(str)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("assignedTo", self.assigned_to)
        writer.write_collection_of_primitive_values("category", self.category)
        writer.write_str_value("closedAt", self.closed_at)
        writer.write_collection_of_primitive_values("costImpact", self.cost_impact)
        writer.write_str_value("createdAt", self.created_at)
        writer.write_collection_of_primitive_values("createdBy", self.created_by)
        writer.write_object_value("customAttributes", self.custom_attributes)
        writer.write_collection_of_primitive_values("discipline", self.discipline)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_collection_of_primitive_values("id", self.id)
        writer.write_bool_value("includeHidden", self.include_hidden)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_collection_of_primitive_values("officialResponseStatus", self.official_response_status)
        writer.write_collection_of_primitive_values("priority", self.priority)
        writer.write_str_value("reference", self.reference)
        writer.write_collection_of_primitive_values("rfiTypeId", self.rfi_type_id)
        writer.write_collection_of_primitive_values("scheduleImpact", self.schedule_impact)
        writer.write_collection_of_primitive_values("status", self.status)
        writer.write_str_value("updatedAt", self.updated_at)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class WithJobGetResponse(Parsable):
    # The account ID.
    account_id: Optional[UUID] = None
    # The date and time the job was completed, presented in ISO 8601 format. If the job has not yet completed, the value is null.
    completed_at: Optional[datetime.datetime] = None
    # The completion status for completed jobs. Possible values: ``success``, ``failed``, ``cancelled``.
    completion_status: Optional[str] = None
    # The date and time the job was created, presented in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The BIM 360 / ACC user ID of the user who created the data request that spawned this job.
    created_by: Optional[str] = None
    # The email address of the user who created the data request that spawned this job.
    created_by_email: Optional[str] = None
    # The end date and time for the data extraction, in ISO 8601 format.This field applies only to schemas that support date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.
    end_date: Optional[str] = None
    # The job ID.
    id: Optional[UUID] = None
    # The last date and time that a user downloaded this job data, in ISO 8601 format.
    last_downloaded_at: Optional[datetime.datetime] = None
    # The ID of the user who last downloaded this job data.
    last_downloaded_by: Optional[str] = None
    # Job progress indicator (0 to 100 percent)
    progress: Optional[int] = None
    # The project ID.
    project_id: Optional[UUID] = None
    # The list of project IDs
    project_id_list: Optional[list[str]] = None
    # The ID of the data request that spawned the job.
    request_id: Optional[UUID] = None
    # Send a notification email when the job completes.
    send_email: Optional[bool] = None
    # The start date and time for the data extraction, in ISO 8601 format.This field applies only to schemas that support date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.
    start_date: Optional[str] = None
    # The date and time the job was started, presented in ISO 8601 format. If the job has not yet started, the value is null.
    started_at: Optional[datetime.datetime] = None
    # The current status of the job. Possible values: ``queued``, ``running``, ``complete``.
    status: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithJobGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithJobGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithJobGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "accountId": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "completedAt": lambda n : setattr(self, 'completed_at', n.get_datetime_value()),
            "completionStatus": lambda n : setattr(self, 'completion_status', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByEmail": lambda n : setattr(self, 'created_by_email', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "lastDownloadedAt": lambda n : setattr(self, 'last_downloaded_at', n.get_datetime_value()),
            "lastDownloadedBy": lambda n : setattr(self, 'last_downloaded_by', n.get_str_value()),
            "progress": lambda n : setattr(self, 'progress', n.get_int_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "projectIdList": lambda n : setattr(self, 'project_id_list', n.get_collection_of_primitive_values(str)),
            "requestId": lambda n : setattr(self, 'request_id', n.get_uuid_value()),
            "sendEmail": lambda n : setattr(self, 'send_email', n.get_bool_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "startedAt": lambda n : setattr(self, 'started_at', n.get_datetime_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("accountId", self.account_id)
        writer.write_datetime_value("completedAt", self.completed_at)
        writer.write_str_value("completionStatus", self.completion_status)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByEmail", self.created_by_email)
        writer.write_str_value("endDate", self.end_date)
        writer.write_uuid_value("id", self.id)
        writer.write_datetime_value("lastDownloadedAt", self.last_downloaded_at)
        writer.write_str_value("lastDownloadedBy", self.last_downloaded_by)
        writer.write_int_value("progress", self.progress)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_collection_of_primitive_values("projectIdList", self.project_id_list)
        writer.write_uuid_value("requestId", self.request_id)
        writer.write_bool_value("sendEmail", self.send_email)
        writer.write_str_value("startDate", self.start_date)
        writer.write_datetime_value("startedAt", self.started_at)
        writer.write_str_value("status", self.status)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class ResponsesGetResponse(Parsable):
    # The type of response. Possible values: ``1`` (Approved), ``2`` (Revise and submit), ``3`` (Rejected).
    category_id: Optional[str] = None
    # The time and date when the response was created.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the response.
    created_by: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the response.
    id: Optional[UUID] = None
    # ``true``: if the response was not deleted.``false``: if the response was deleted.
    is_active: Optional[bool] = None
    # ``true``: settings this response for a submittal item means an approval.``false``: settings this response for a submittal item means dis-approval.This attribute is taken from the related categoryId
    is_approval: Optional[bool] = None
    # ``true``: if the response is currently associated with a submittal item.``false``: if the response is not currently associated with a submittal item.
    is_in_use: Optional[bool] = None
    # Not relevant
    key: Optional[str] = None
    # Not relevant
    platform_id: Optional[str] = None
    # The time and date when the response was last updated.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the response.
    updated_by: Optional[str] = None
    # The content of the response.
    value: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResponsesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResponsesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResponsesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "categoryId": lambda n : setattr(self, 'category_id', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "isApproval": lambda n : setattr(self, 'is_approval', n.get_bool_value()),
            "isInUse": lambda n : setattr(self, 'is_in_use', n.get_bool_value()),
            "key": lambda n : setattr(self, 'key', n.get_str_value()),
            "platformId": lambda n : setattr(self, 'platform_id', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "value": lambda n : setattr(self, 'value', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("categoryId", self.category_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_bool_value("isApproval", self.is_approval)
        writer.write_bool_value("isInUse", self.is_in_use)
        writer.write_str_value("key", self.key)
        writer.write_str_value("platformId", self.platform_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("value", self.value)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ActionHistoriesGetResponse_results_options(Parsable):
    """
    Additional context for the action. The fields present in this object depend on the action history type.For Approval workflows, the object includes:``stepName`` – the name of the workflow step.``taskDefinitionKey`` – the key that identifies the workflow task.``workflowInstanceId`` – the ID of the workflow instance.These fields provide task-level process information.For Normal workflows, the object may include fields such as to and from that indicate a change in status.
    """
    # The name of the workflow step where the action occurred.
    step_name: Optional[str] = None
    # The key that identifies the specific workflow task.
    task_definition_key: Optional[str] = None
    # The ID of the workflow instance the action belongs to.
    workflow_instance_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ActionHistoriesGetResponse_results_options:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ActionHistoriesGetResponse_results_options
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ActionHistoriesGetResponse_results_options()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "stepName": lambda n : setattr(self, 'step_name', n.get_str_value()),
            "taskDefinitionKey": lambda n : setattr(self, 'task_definition_key', n.get_str_value()),
            "workflowInstanceId": lambda n : setattr(self, 'workflow_instance_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("stepName", self.step_name)
        writer.write_str_value("taskDefinitionKey", self.task_definition_key)
        writer.write_str_value("workflowInstanceId", self.workflow_instance_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_permitted_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_permittedAttributes
    from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_required_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_requiredAttributes
    from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_status import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status

@dataclass
class RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS(Parsable):
    # The maximum number of users that can be assigned to this RFI.
    max_assignees: Optional[int] = None
    # The list of attributes that the user is optionally allowed to include when updating or creating the RFI in the specified status.
    permitted_attributes: Optional[list[RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_permittedAttributes]] = None
    # The list of attributes that are required when creating or updating an RFI.
    required_attributes: Optional[list[RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_requiredAttributes]] = None
    # The current response status of the RFI for single-reviewer workflows (US):Possible values:``draft``, ``submitted``, ``open``, ``answered``, ``rejected``, ``closed``, ``void``.For more information about workflows, see `About RFI Workflows – Autodesk Help <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Types>`_.
    status: Optional[RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_permitted_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_permittedAttributes
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_required_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_requiredAttributes
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_status import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status

        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_permitted_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_permittedAttributes
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_required_attributes import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_requiredAttributes
        from .rfis_post_response_permitted_actions_update_rfi_permitted_statuses_wf_u_s_status import RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status

        fields: dict[str, Callable[[Any], None]] = {
            "maxAssignees": lambda n : setattr(self, 'max_assignees', n.get_int_value()),
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_object_values(RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_permittedAttributes)),
            "requiredAttributes": lambda n : setattr(self, 'required_attributes', n.get_collection_of_object_values(RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_requiredAttributes)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(RfisPostResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("maxAssignees", self.max_assignees)
        writer.write_collection_of_object_values("permittedAttributes", self.permitted_attributes)
        writer.write_collection_of_object_values("requiredAttributes", self.required_attributes)
        writer.write_enum_value("status", self.status)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_response_patch_response_response import WithResponsePatchResponse_response
    from .with_response_patch_response_response_attachments import WithResponsePatchResponse_responseAttachments
    from .with_response_patch_response_rfi import WithResponsePatchResponse_rfi

@dataclass
class WithResponsePatchResponse(Parsable):
    # The details of the response that was created or updated.
    response: Optional[WithResponsePatchResponse_response] = None
    # The list of attachment groups created as part of the response.
    response_attachments: Optional[list[WithResponsePatchResponse_responseAttachments]] = None
    # The details of the RFI to which the response was added.
    rfi: Optional[list[WithResponsePatchResponse_rfi]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_response_response import WithResponsePatchResponse_response
        from .with_response_patch_response_response_attachments import WithResponsePatchResponse_responseAttachments
        from .with_response_patch_response_rfi import WithResponsePatchResponse_rfi

        from .with_response_patch_response_response import WithResponsePatchResponse_response
        from .with_response_patch_response_response_attachments import WithResponsePatchResponse_responseAttachments
        from .with_response_patch_response_rfi import WithResponsePatchResponse_rfi

        fields: dict[str, Callable[[Any], None]] = {
            "response": lambda n : setattr(self, 'response', n.get_object_value(WithResponsePatchResponse_response)),
            "responseAttachments": lambda n : setattr(self, 'response_attachments', n.get_collection_of_object_values(WithResponsePatchResponse_responseAttachments)),
            "rfi": lambda n : setattr(self, 'rfi', n.get_collection_of_object_values(WithResponsePatchResponse_rfi)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("response", self.response)
        writer.write_collection_of_object_values("responseAttachments", self.response_attachments)
        writer.write_collection_of_object_values("rfi", self.rfi)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .attributes.attributes_request_builder import AttributesRequestBuilder
    from .rfis.rfis_request_builder import RfisRequestBuilder
    from .rfi_types.rfi_types_request_builder import RfiTypesRequestBuilder
    from .search_rfis.search_rfis_request_builder import SearchRfisRequestBuilder
    from .users.users_request_builder import UsersRequestBuilder
    from .workflow.workflow_request_builder import WorkflowRequestBuilder

class WithProjectItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/rfis/v3/projects/{projectId}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithProjectItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/rfis/v3/projects/{projectId}", path_parameters)
    
    @property
    def attributes(self) -> AttributesRequestBuilder:
        """
        The attributes property
        """
        from .attributes.attributes_request_builder import AttributesRequestBuilder

        return AttributesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rfi_types(self) -> RfiTypesRequestBuilder:
        """
        The rfiTypes property
        """
        from .rfi_types.rfi_types_request_builder import RfiTypesRequestBuilder

        return RfiTypesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def rfis(self) -> RfisRequestBuilder:
        """
        The rfis property
        """
        from .rfis.rfis_request_builder import RfisRequestBuilder

        return RfisRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def search_rfis(self) -> SearchRfisRequestBuilder:
        """
        The searchRfis property
        """
        from .search_rfis.search_rfis_request_builder import SearchRfisRequestBuilder

        return SearchRfisRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def users(self) -> UsersRequestBuilder:
        """
        The users property
        """
        from .users.users_request_builder import UsersRequestBuilder

        return UsersRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def workflow(self) -> WorkflowRequestBuilder:
        """
        The workflow property
        """
        from .workflow.workflow_request_builder import WorkflowRequestBuilder

        return WorkflowRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .review_sheets_get_response_results_processing_state import ReviewSheetsGetResponse_results_processingState

@dataclass
class ReviewSheetsGetResponse_results(Parsable):
    # - ``true`` if the review sheet has been deleted.- ``false`` if the review sheet has not been deleted.Note that if the review sheet has been deleted, it will not be published.
    deleted: Optional[bool] = None
    # The source file name of the review sheet.
    file_name: Optional[str] = None
    # The ID of the review sheet.
    id: Optional[str] = None
    # The number of the review sheet.
    number: Optional[str] = None
    # The page number of the source file from which the review sheet was generated.
    page: Optional[float] = None
    # The processing state of the review sheet. Possible values:- ``PROCESSING``: the review sheet is being processed.- ``AUDITING``: the review sheet is being audited.- ``ROTATING``: the review sheet is being rotated.- ``READY``: the review sheet is ready for updating or publishing.- ``FAILED``: the processing of the review sheet failed.- ``PUBLISHING`` the review sheet is publishing.
    processing_state: Optional[ReviewSheetsGetResponse_results_processingState] = None
    # The rotation of the review sheet. Possible values: ``0``, ``90``, ``180``, ``270``.
    rotation: Optional[float] = None
    # The tags of the review sheet.
    tags: Optional[list[str]] = None
    # The title of the review sheet.
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewSheetsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewSheetsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewSheetsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .review_sheets_get_response_results_processing_state import ReviewSheetsGetResponse_results_processingState

        from .review_sheets_get_response_results_processing_state import ReviewSheetsGetResponse_results_processingState

        fields: dict[str, Callable[[Any], None]] = {
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "page": lambda n : setattr(self, 'page', n.get_float_value()),
            "processingState": lambda n : setattr(self, 'processing_state', n.get_enum_value(ReviewSheetsGetResponse_results_processingState)),
            "rotation": lambda n : setattr(self, 'rotation', n.get_float_value()),
            "tags": lambda n : setattr(self, 'tags', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("deleted", self.deleted)
        writer.write_str_value("fileName", self.file_name)
        writer.write_str_value("id", self.id)
        writer.write_str_value("number", self.number)
        writer.write_float_value("page", self.page)
        writer.write_enum_value("processingState", self.processing_state)
        writer.write_float_value("rotation", self.rotation)
        writer.write_collection_of_primitive_values("tags", self.tags)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .attachments_get_response import AttachmentsGetResponse
    from .attachments_post_request_body import AttachmentsPostRequestBody
    from .attachments_post_response import AttachmentsPostResponse
    from .item.with_attachment_item_request_builder import WithAttachmentItemRequestBuilder

class AttachmentsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/items/{itemId}/attachments
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new AttachmentsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}/items/{itemId}/attachments{?filter%5BcategoryId%5D*,filter%5BisFileUploaded%5D*,filter%5Brevision%5D*,limit*,offset*,sort*}", path_parameters)
    
    def by_attachment_id(self,attachment_id: str) -> WithAttachmentItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.submittals.v2.projects.item.items.item.attachments.item collection
        param attachment_id: The ID of the attachment. To find the ID, call `GET attachments </en/docs/acc/v1/reference/http/submittals-items-itemId-attachments-GET/>`_.
        Returns: WithAttachmentItemRequestBuilder
        """
        if attachment_id is None:
            raise TypeError("attachment_id cannot be null.")
        from .item.with_attachment_item_request_builder import WithAttachmentItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["attachmentId"] = attachment_id
        return WithAttachmentItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[AttachmentsRequestBuilderGetQueryParameters]] = None) -> Optional[AttachmentsGetResponse]:
        """
        Retrieves attachments for a specified submittal item. Note that this endpoint is not compatible with BIM 360 projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[AttachmentsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .attachments_get_response import AttachmentsGetResponse

        return await self.request_adapter.send_async(request_info, AttachmentsGetResponse, None)
    
    async def post(self,body: AttachmentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[AttachmentsPostResponse]:
        """
        Adds an attachment to a submittal item within a project. This allows users to include relevant files as part of the submittal process.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[AttachmentsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .attachments_post_response import AttachmentsPostResponse

        return await self.request_adapter.send_async(request_info, AttachmentsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[AttachmentsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves attachments for a specified submittal item. Note that this endpoint is not compatible with BIM 360 projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: AttachmentsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Adds an attachment to a submittal item within a project. This allows users to include relevant files as part of the submittal process.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> AttachmentsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: AttachmentsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return AttachmentsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class AttachmentsRequestBuilderGetQueryParameters():
        """
        Retrieves attachments for a specified submittal item. Note that this endpoint is not compatible with BIM 360 projects.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtercategory_id":
                return "filter%5BcategoryId%5D"
            if original_name == "filteris_file_uploaded":
                return "filter%5BisFileUploaded%5D"
            if original_name == "filterrevision":
                return "filter%5Brevision%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter attachments by category identifier. Multiple values can be separated by commas.
        filtercategory_id: Optional[str] = None

        # ``true``: to filter files that are uploaded.``false``: to filter files that are not uploaded.
        filteris_file_uploaded: Optional[str] = None

        # Filter items with the specified revision number. You can specify multiple values. Separate multiple values with commas. For example, ``filter[revision]=1``.
        filterrevision: Optional[str] = None

        # The maximum number of results per page. Possible values: ``1``- ``50``. Default value: ``20``. For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The number of results to skip before starting to return data. For example, to skip the first 20 results, include ``offset=20`` in the query string. For more details, see the `JSON API Paging Help documentation <https://jsonapi.org/format/#fetching-pagination>`_.
        offset: Optional[int] = None

        # Sort attachments by specific fields in either ascending (``asc``) or descending (``desc``) order. Separate multiple criteria with commas. For example, ``statusId asc``.Possible values: ``id``, ``isFileUploaded``, ``asyncState``, ``createdBy``, ``createdAt``, ``updatedBy``, ``updatedAt``, ``name``, ``uploadUrn``, ``urn``, ``urnPage``, ``resourceUrns``, ``urnTypeId``, ``urnVersion``, ``revision``, ``categoryId``, ``isReview``, ``isResponse``.
        sort: Optional[str] = None

    
    @dataclass
    class AttachmentsRequestBuilderGetRequestConfiguration(RequestConfiguration[AttachmentsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class AttachmentsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ExpensesPostResponse(Parsable):
    # Not relevant
    aggregate_by: Optional[str] = None
    # The total value of the subitems of the expense.
    amount: Optional[float] = None
    # The date and time when the expense was approved.
    approved_at: Optional[str] = None
    # The ID of the Budget Pay App the expense amount will be aggregated to.
    budget_payment_id: Optional[str] = None
    # The user who made the last change to the expense. This is the ID of a user managed by BIM 360 Admin.
    changed_by: Optional[str] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The user who created the expense. This is the ID of a user managed by BIM 360 Admin.
    creator_id: Optional[str] = None
    # The detailed description of the expense.Max length: 2048
    description: Optional[str] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The date and time determine which distribution period this expense belongs to, after the status of ``status`` is set to ``approved`` or ``paid``.
    forecast_distribution_at: Optional[str] = None
    # The unique identifier of the expense.
    id: Optional[str] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # The date and time when the expense was issued.
    issued_at: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The ID of the Main Contract to which the expense belongs.
    main_contract_id: Optional[str] = None
    # The name of the expense.Max length: 1024
    name: Optional[str] = None
    # The note attached to the expense, if returned. The note consists of Tiptap formatted rich text. For more information, see `https://tiptap.dev <https://tiptap.dev/introduction/>`_.
    note: Optional[str] = None
    # The auto-generated sequence number for the expense.Max length: 255
    number: Optional[str] = None
    # The date and time when the expense was paid.
    paid_at: Optional[str] = None
    # The payment due date and time of the expense.
    payment_due: Optional[str] = None
    # The user who purchased the expense. This is the ID of a user managed by BIM 360 Admin.
    purchased_by: Optional[str] = None
    # The date and time when the expense was received.
    received_at: Optional[str] = None
    # The user-provided reference number of the expense.
    reference_number: Optional[str] = None
    # The applicable scope of the expense. Possible values: ``full``, ``partial``.
    scope: Optional[str] = None
    # The status of the expense. Possible values: ``draft``, ``pending``, ``revise``, ``rejected``, ``approved``, ``paid``.
    status: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    supplier_company_uid: Optional[str] = None
    # The ID of a company managed by a BIM 360 Admin. Detailed company information can be retrieved by calling `GET projects/:project_id/companies <en/docs/bim360/v1/reference/http/projects-:project_id-companies-GET/>`_ and locating the `member_group_id` in the response.
    supplier_id: Optional[str] = None
    # The supplier company name for the expense. Any company can be used regardless the ones added in Account Administration.Max length: 255
    supplier_name: Optional[str] = None
    # The term of the expense. This is customizable by the project administrator.
    term: Optional[str] = None
    # The type of the expense. This is customizable by the project administrator.Max length: 128
    type: Optional[str] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ExpensesPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ExpensesPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ExpensesPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "aggregateBy": lambda n : setattr(self, 'aggregate_by', n.get_str_value()),
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "approvedAt": lambda n : setattr(self, 'approved_at', n.get_str_value()),
            "budgetPaymentId": lambda n : setattr(self, 'budget_payment_id', n.get_str_value()),
            "changedBy": lambda n : setattr(self, 'changed_by', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "creatorId": lambda n : setattr(self, 'creator_id', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "forecastDistributionAt": lambda n : setattr(self, 'forecast_distribution_at', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "issuedAt": lambda n : setattr(self, 'issued_at', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "mainContractId": lambda n : setattr(self, 'main_contract_id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "note": lambda n : setattr(self, 'note', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "paidAt": lambda n : setattr(self, 'paid_at', n.get_str_value()),
            "paymentDue": lambda n : setattr(self, 'payment_due', n.get_str_value()),
            "purchasedBy": lambda n : setattr(self, 'purchased_by', n.get_str_value()),
            "receivedAt": lambda n : setattr(self, 'received_at', n.get_str_value()),
            "referenceNumber": lambda n : setattr(self, 'reference_number', n.get_str_value()),
            "scope": lambda n : setattr(self, 'scope', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_str_value()),
            "supplierCompanyUid": lambda n : setattr(self, 'supplier_company_uid', n.get_str_value()),
            "supplierId": lambda n : setattr(self, 'supplier_id', n.get_str_value()),
            "supplierName": lambda n : setattr(self, 'supplier_name', n.get_str_value()),
            "term": lambda n : setattr(self, 'term', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("aggregateBy", self.aggregate_by)
        writer.write_float_value("amount", self.amount)
        writer.write_str_value("approvedAt", self.approved_at)
        writer.write_str_value("budgetPaymentId", self.budget_payment_id)
        writer.write_str_value("changedBy", self.changed_by)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("creatorId", self.creator_id)
        writer.write_str_value("description", self.description)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_str_value("forecastDistributionAt", self.forecast_distribution_at)
        writer.write_str_value("id", self.id)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_str_value("issuedAt", self.issued_at)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_str_value("mainContractId", self.main_contract_id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("note", self.note)
        writer.write_str_value("number", self.number)
        writer.write_str_value("paidAt", self.paid_at)
        writer.write_str_value("paymentDue", self.payment_due)
        writer.write_str_value("purchasedBy", self.purchased_by)
        writer.write_str_value("receivedAt", self.received_at)
        writer.write_str_value("referenceNumber", self.reference_number)
        writer.write_str_value("scope", self.scope)
        writer.write_str_value("status", self.status)
        writer.write_str_value("supplierCompanyUid", self.supplier_company_uid)
        writer.write_str_value("supplierId", self.supplier_id)
        writer.write_str_value("supplierName", self.supplier_name)
        writer.write_str_value("term", self.term)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from enum import Enum

class GetSortQueryParameterType(str, Enum):
    Asc = "asc",
    Desc = "desc",
    Name = "name",
    Description = "description",
    UpdatedAt = "updatedAt",
    ApprovalStatus = "approvalStatus",
    Version = "version",
    SortNameDesc = "sort=name desc",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .relationships_writable_get_response_entity_types import RelationshipsWritableGetResponse_entityTypes

@dataclass
class RelationshipsWritableGetResponse(Parsable):
    # The domain to which the entity types belong.For example: ``autodesk-bim360-asset``To learn more about domains and entities, see the `Relationship Service Field Guide </en/docs/bim360/v1/overview/field-guide/relationships>`_.
    domain: Optional[str] = None
    # The list of entity types in the domain.
    entity_types: Optional[list[RelationshipsWritableGetResponse_entityTypes]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RelationshipsWritableGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RelationshipsWritableGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RelationshipsWritableGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .relationships_writable_get_response_entity_types import RelationshipsWritableGetResponse_entityTypes

        from .relationships_writable_get_response_entity_types import RelationshipsWritableGetResponse_entityTypes

        fields: dict[str, Callable[[Any], None]] = {
            "domain": lambda n : setattr(self, 'domain', n.get_str_value()),
            "entityTypes": lambda n : setattr(self, 'entity_types', n.get_collection_of_object_values(RelationshipsWritableGetResponse_entityTypes)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("domain", self.domain)
        writer.write_collection_of_object_values("entityTypes", self.entity_types)
    




from enum import Enum

class SmartregisterGetResponse_targetGroup(str, Enum):
    ACTIONANDINFORMATIONAL = "ACTION AND INFORMATIONAL",
    CloseoutSubmittals = "Closeout Submittals",
    DIVISION01REQUIREMENTS = "DIVISION 01 REQUIREMENTS",
    FieldQualityControl = "Field Quality Control",
    Mockups = "Mockups",
    QUALITYASSURANCE = "QUALITY ASSURANCE",
    TESTSANDINSPECTIONS = "TESTS AND INSPECTIONS",

